@@ -168,7 +168,7 @@ func TestWebSocketViaActivator(t *testing.T) {
 	}
 
 	// Wait for the activator endpoints to equalize.
-	if err := waitForActivatorEndpoints(&testContext{
+	if err := waitForActivatorEndpoints(&TestContext{
 		t:         t,
 		resources: resources,
 		clients:   clients,
@@ -275,7 +275,7 @@ func TestWebSocketBlueGreenRoute(t *testing.T) {
 	)
 	resps, err := webSocketResponseFreqs(t, clients, tealURL, numReqs)
 	if err != nil {
-		t.Errorf("Failed to send and receive websocket messages: %v", err)
+		t.Error("Failed to send and receive websocket messages:", err)
 	}
 	if len(resps) != 2 {
 		t.Errorf("Number of responses: %d, want: 2", len(resps))
